/* SPDX-FileCopyrightText: 2020 - Sébastien Wilmet <swilmet@gnome.org>
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */

#ifndef TEPL_WINDOW_ACTIONS_SEARCH_H
#define TEPL_WINDOW_ACTIONS_SEARCH_H

#include "tepl-application-window.h"

G_BEGIN_DECLS

G_GNUC_INTERNAL
void	_tepl_window_actions_search_add_actions	(TeplApplicationWindow *tepl_window);

G_END_DECLS

#endif /* TEPL_WINDOW_ACTIONS_SEARCH_H */
